/* config.c - 2000/06/21 */
/*
 *  EasyTAG - Tag editor for MP3 and OGG files
 *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>

#include "setting.h"
#include "prefs.h"
#include "bar.h"
#include "easytag.h"
#include "charset.h"
#include "scan.h"
#include "misc.h"
#include "mylocale.h"


/***************
 * Declaration *
 ***************/

/* Base directory created into home dir */
gchar *EASYTAG_DIR                          = ".easytag/";
/* File for configuration */
gchar *CONFIG_FILE                          = ".easytag/easytagrc";
/* File of masks for tag scanner */
gchar *SCAN_TAG_MASKS_FILE                  = ".easytag/scan_tag.mask";
/* File of masks for rename file scanner */
gchar *RENAME_FILE_MASKS_FILE                = ".easytag/rename_file.mask";
/* File for history of DefaultPathToMp3 combobox */
gchar *DEFAULT_PATH_TO_MP3_HISTORY_FILE      = ".easytag/default_path_to_mp3.history";
/* File for history of DefaultComment combobox */
gchar *DEFAULT_TAG_COMMENT_HISTORY_FILE      = ".easytag/default_tag_comment.history";
/* File for history of TranslationTablePath combobox */
gchar *TRANSLATION_TABLE_PATH_HISTORY_FILE   = ".easytag/translation_table_path.history";
/* File for history of BrowserEntry combobox */
gchar *PATH_ENTRY_HISTORY_FILE               = ".easytag/browser_path.history";
/* File for history of PlayListNameEntry combobox */
gchar *PLAY_LIST_NAME_HISTORY_FILE           = ".easytag/play_list_name.history";
/* File for history of run program combobox */
gchar *RUN_PROGRAM_HISTORY_FILE              = ".easytag/run_program.history";


/**************
 * Prototypes *
 **************/



/********************
 * Config Variables *
 ********************/
tConfigVariable Config_Variables[] =
{
    {"load_on_startup",                CV_TYPE_BOOL,    &LOAD_ON_STARTUP            },
    {"default_path_to_mp3",            CV_TYPE_STRING,  &DEFAULT_PATH_TO_MP3        },
    {"sorting_file_mode",              CV_TYPE_BOOL,    &SORTING_FILE_MODE          },
    {"write_id3v1_tag",                CV_TYPE_BOOL,    &WRITE_ID3V1_TAG            },
    {"write_id3v2_tag",                CV_TYPE_BOOL,    &WRITE_ID3V2_TAG            },
    {"strip_tag_when_empty_fields",    CV_TYPE_BOOL,    &STRIP_TAG_WHEN_EMPTY_FIELDS},
    {"browse_subdir",                  CV_TYPE_BOOL,    &BROWSE_SUBDIR              },
    {"open_selected_browser_node",     CV_TYPE_BOOL,    &OPEN_SELECTED_BROWSER_NODE },
    {"keep_tree_browser_in_memory",    CV_TYPE_BOOL,    &KEEP_TREE_BROWSER_IN_MEMORY},
    {"save_config_before_exit",        CV_TYPE_BOOL,    &SAVE_CONFIG_BEFORE_EXIT    },

    {"set_main_window_size",           CV_TYPE_BOOL,    &SET_MAIN_WINDOW_SIZE       },
    {"main_window_height",             CV_TYPE_INT,     &MAIN_WINDOW_HEIGHT         },
    {"main_window_width",              CV_TYPE_INT,     &MAIN_WINDOW_WIDTH          },
    {"set_pane_handle_position1",      CV_TYPE_INT,     &SET_PANE_HANDLE_POSITION1  },
    {"pane_handle_position1",          CV_TYPE_INT,     &PANE_HANDLE_POSITION1      },
    {"set_pane_handle_position2",      CV_TYPE_INT,     &SET_PANE_HANDLE_POSITION2  },
    {"pane_handle_position2",          CV_TYPE_INT,     &PANE_HANDLE_POSITION2      },
    {"show_header_infos",              CV_TYPE_BOOL,    &SHOW_HEADER_INFO           },
    {"show_command_buttons",           CV_TYPE_BOOL,    &SHOW_COMMAND_BUTTONS       },

    {"show_spin_button",               CV_TYPE_BOOL,    &SHOW_SPIN_BUTTON        },
    {"show_go_button",                 CV_TYPE_BOOL,    &SHOW_SPIN_BUTTON        },
    {"show_first_button",              CV_TYPE_BOOL,    &SHOW_FIRST_BUTTON       },
    {"show_prev_button",               CV_TYPE_BOOL,    &SHOW_PREV_BUTTON        },
    {"show_next_button",               CV_TYPE_BOOL,    &SHOW_NEXT_BUTTON        },
    {"show_last_button",               CV_TYPE_BOOL,    &SHOW_LAST_BUTTON        },
    {"show_scan_button",               CV_TYPE_BOOL,    &SHOW_SCAN_BUTTON        },
    {"show_scan_all_button",           CV_TYPE_BOOL,    &SHOW_SCAN_ALL_BUTTON    },
    {"show_remove_button",             CV_TYPE_BOOL,    &SHOW_REMOVE_BUTTON      },
    {"show_remove_all_button",         CV_TYPE_BOOL,    &SHOW_REMOVE_ALL_BUTTON  },
    {"show_undo_button",               CV_TYPE_BOOL,    &SHOW_UNDO_BUTTON        },
    {"show_undo_all_button",           CV_TYPE_BOOL,    &SHOW_UNDO_ALL_BUTTON    },
    {"show_redo_button",               CV_TYPE_BOOL,    &SHOW_REDO_BUTTON        },
    {"show_redo_all_button",           CV_TYPE_BOOL,    &SHOW_REDO_ALL_BUTTON    },
    {"show_save_button",               CV_TYPE_BOOL,    &SHOW_SAVE_BUTTON        },
    {"show_save_all_button",           CV_TYPE_BOOL,    &SHOW_SAVE_ALL_BUTTON    },
    {"show_quit_button",               CV_TYPE_BOOL,    &SHOW_QUIT_BUTTON        },

    {"use_colored_tooltips",           CV_TYPE_BOOL,    &USE_COLORED_TOOLTIPS    },

    {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION           },
    {"number_track_formated",                     CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED     },
    {"replace_illegal_character_in_filename",CV_TYPE_BOOL,    &REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME},

    {"use_character_set_translation",        CV_TYPE_BOOL,    &USE_CHARACTER_SET_TRANSLATION  },
    {"file_character_set",                   CV_TYPE_STRING,  &FILE_CHARACTER_SET             },
    {"user_character_set",                   CV_TYPE_STRING,  &USER_CHARACTER_SET             },

    {"message_box_position_none",            CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_NONE      },
    {"message_box_position_center",          CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_CENTER    },
    {"message_box_position_mouse",           CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_MOUSE     },

    {"convert_underscore_and_p20_into_space",CV_TYPE_BOOL,    &CONVERT_UNDERSCORE_AND_P20_INTO_SPACE},
    {"convert_space_into_underscore",        CV_TYPE_BOOL,    &CONVERT_SPACE_INTO_UNDERSCORE        },
    {"overwrite_tag_field",                  CV_TYPE_BOOL,    &OVERWRITE_TAG_FIELD                  },
    {"set_default_comment",                  CV_TYPE_BOOL,    &SET_DEFAULT_COMMENT                  },
    {"default_comment",                      CV_TYPE_STRING,  &DEFAULT_COMMENT                      },
    {"scan_window_on_top",                   CV_TYPE_BOOL,    &SCAN_WINDOW_ON_TOP                   },
    {"set_scanner_window_position",          CV_TYPE_BOOL,    &SET_SCANNER_WINDOW_POSITION          },
    {"scanner_window_x",                     CV_TYPE_INT,     &SCANNER_WINDOW_X                     },
    {"scanner_window_y",                     CV_TYPE_INT,     &SCANNER_WINDOW_Y                     },

    {"confirm_before_exit",                  CV_TYPE_BOOL,    &CONFIRM_BEFORE_EXIT                  },
    {"confirm_write_tag",                    CV_TYPE_BOOL,    &CONFIRM_WRITE_TAG                    },
    {"confirm_rename_file",                  CV_TYPE_BOOL,    &CONFIRM_RENAME_FILE                  },
    {"confirm_write_playlist",               CV_TYPE_BOOL,    &CONFIRM_WRITE_PLAYLIST               },
    {"options_notebook_page",                CV_TYPE_INT,     &OPTIONS_NOTEBOOK_PAGE                },
    {"process_filename_field",               CV_TYPE_BOOL,    &PROCESS_FILENAME_FIELD               },
    {"process_title_field",                  CV_TYPE_BOOL,    &PROCESS_TITLE_FIELD                  },
    {"process_artist_field",                 CV_TYPE_BOOL,    &PROCESS_ARTIST_FIELD                 },
    {"process_album_field",                  CV_TYPE_BOOL,    &PROCESS_ALBUM_FIELD                  },
    {"process_comment_field",                CV_TYPE_BOOL,    &PROCESS_COMMENT_FIELD                },
    {"pf_convert_into_space",                CV_TYPE_BOOL,    &PF_CONVERT_INTO_SPACE                },
    {"pf_convert_space",                     CV_TYPE_BOOL,    &PF_CONVERT_SPACE                     },
    {"pf_convert_all_uppercase",             CV_TYPE_BOOL,    &PF_CONVERT_ALL_UPPERCASE             },
    {"pf_convert_all_downcase",              CV_TYPE_BOOL,    &PF_CONVERT_ALL_DOWNCASE              },
    {"pf_convert_first_letter_uppercase",    CV_TYPE_BOOL,    &PF_CONVERT_FIRST_LETTER_UPPERCASE    },
    {"pf_convert_first_letters_uppercase",   CV_TYPE_BOOL,    &PF_CONVERT_FIRST_LETTERS_UPPERCASE   },
    {"pf_remove_space",                      CV_TYPE_BOOL,    &PF_REMOVE_SPACE                      },
    {"pf_insert_space",                      CV_TYPE_BOOL,    &PF_INSERT_SPACE                      },
    {"pf_only_one_space",                    CV_TYPE_BOOL,    &PF_ONLY_ONE_SPACE                    },

    {"playlist_name",                        CV_TYPE_STRING,  &PLAYLIST_NAME                        },
    {"playlist_use_name",                    CV_TYPE_BOOL,    &PLAYLIST_USE_NAME                    },
    {"playlist_use_dir_name",                CV_TYPE_BOOL,    &PLAYLIST_USE_DIR_NAME                },
    {"playlist_full_path",                   CV_TYPE_BOOL,    &PLAYLIST_FULL_PATH                   },
    {"playlist_relative_path",               CV_TYPE_BOOL,    &PLAYLIST_RELATIVE_PATH               }
};




/*************
 * Functions *
 *************/

/*
 * Define and Load default values into config variables
 */
void Init_Config_Variables (void)
{
    /*
     * Common
     */
    LOAD_ON_STARTUP              = 1;
    DEFAULT_PATH_TO_MP3          = g_strdup(HOME_VARIABLE);
    SORTING_FILE_MODE            = SORTING_BY_ASCENDING_FILENAME;
    WRITE_ID3V1_TAG              = 1;
    WRITE_ID3V2_TAG              = 1;
    STRIP_TAG_WHEN_EMPTY_FIELDS  = 1;
    BROWSE_SUBDIR                = 1;
    OPEN_SELECTED_BROWSER_NODE   = 1;
    KEEP_TREE_BROWSER_IN_MEMORY  = 0;
    SAVE_CONFIG_BEFORE_EXIT      = 0;

    /*
     * User Interface
     */
    SET_MAIN_WINDOW_SIZE      = 1;
    MAIN_WINDOW_HEIGHT        = -1;
    MAIN_WINDOW_WIDTH         = 800;
    SET_PANE_HANDLE_POSITION1 = 1;
    PANE_HANDLE_POSITION1     = 490;
    SET_PANE_HANDLE_POSITION2 = 1;
    PANE_HANDLE_POSITION2     = 230;
    SHOW_HEADER_INFO          = 1;
    SHOW_COMMAND_BUTTONS      = 0;

    SHOW_SPIN_BUTTON       = 1;
    SHOW_GO_BUTTON         = 1;
    SHOW_FIRST_BUTTON      = 1;
    SHOW_PREV_BUTTON       = 1;
    SHOW_NEXT_BUTTON       = 1;
    SHOW_LAST_BUTTON       = 1;
    SHOW_SCAN_BUTTON       = 1;
    SHOW_SCAN_ALL_BUTTON   = 1;
    SHOW_REMOVE_BUTTON     = 1;
    SHOW_REMOVE_ALL_BUTTON = 1;
    SHOW_UNDO_BUTTON       = 1;
    SHOW_UNDO_ALL_BUTTON   = 1;
    SHOW_REDO_BUTTON       = 1;
    SHOW_REDO_ALL_BUTTON   = 1;
    SHOW_SAVE_BUTTON       = 1;
    SHOW_SAVE_ALL_BUTTON   = 1;
    SHOW_QUIT_BUTTON       = 1;
    
    USE_COLORED_TOOLTIPS   = 0;

    /*
     * Scanner
     */
    CONVERT_UNDERSCORE_AND_P20_INTO_SPACE = 1;
    CONVERT_SPACE_INTO_UNDERSCORE         = 0;
    OVERWRITE_TAG_FIELD                   = 0;
    SET_DEFAULT_COMMENT                   = 0;
    DEFAULT_COMMENT                       = g_strdup("Tagged with EasyTAG");
    SCAN_WINDOW_ON_TOP                    = 1;
    SET_SCANNER_WINDOW_POSITION           = 0;
    SCANNER_WINDOW_X                      = 11;
    SCANNER_WINDOW_Y                      = 155;

    /*
     * Misc
     */
    REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = 1;
    DATE_AUTO_COMPLETION          = 1;
    NUMBER_TRACK_FORMATED         = 1;
    USE_CHARACTER_SET_TRANSLATION = 0;
    //TRANSLATION_TABLE_PATH      = NULL;
    FILE_CHARACTER_SET            = g_strdup("UTF-8");
    USER_CHARACTER_SET            = g_strdup("ISO-8859-15");

    MESSAGE_BOX_POSITION_NONE     = 0;
    MESSAGE_BOX_POSITION_CENTER   = 1;
    MESSAGE_BOX_POSITION_MOUSE    = 0;

    /*
     * Confirmation
     */
    CONFIRM_BEFORE_EXIT    = 1;
    CONFIRM_WRITE_TAG      = 1;
    CONFIRM_RENAME_FILE    = 1;
    CONFIRM_WRITE_PLAYLIST = 1;

    /*
     * Scanner window
     */
    PROCESS_FILENAME_FIELD             = 0;
    PROCESS_TITLE_FIELD                = 1;
    PROCESS_ARTIST_FIELD               = 1;
    PROCESS_ALBUM_FIELD                = 1;
    PROCESS_COMMENT_FIELD              = 1;
    PF_CONVERT_INTO_SPACE              = 1;
    PF_CONVERT_SPACE                   = 0;
    PF_CONVERT_ALL_UPPERCASE           = 0;
    PF_CONVERT_ALL_DOWNCASE            = 0;
    PF_CONVERT_FIRST_LETTER_UPPERCASE  = 0;
    PF_CONVERT_FIRST_LETTERS_UPPERCASE = 1;
    PF_REMOVE_SPACE                    = 0;
    PF_INSERT_SPACE                    = 0;
    PF_ONLY_ONE_SPACE                  = 1;

    /*
     * Playlist
     */
    PLAYLIST_NAME          = g_strdup("playlist");
    PLAYLIST_USE_NAME      = 0;
    PLAYLIST_USE_DIR_NAME  = 1;
    PLAYLIST_FULL_PATH     = 0;
    PLAYLIST_RELATIVE_PATH = 1;
    
    /*
     * Other parameters
     */
    OPTIONS_NOTEBOOK_PAGE = 0;
}


/*
 * Put changes in config variables
 */
void Apply_Changes (void)
{
    if (OptionsWindow)
    {
        /*
         * Changes to config file
         */
        /* Common */
        LOAD_ON_STARTUP             = GTK_TOGGLE_BUTTON(LoadOnStartup)->active;
        DEFAULT_PATH_TO_MP3         = g_strdup(gtk_entry_get_text_1(DefaultPathToMp3));
        WRITE_ID3V1_TAG             = GTK_TOGGLE_BUTTON(WriteId3v1Tag)->active;
        WRITE_ID3V2_TAG             = GTK_TOGGLE_BUTTON(WriteId3v2Tag)->active;
        STRIP_TAG_WHEN_EMPTY_FIELDS = GTK_TOGGLE_BUTTON(StripTagWhenEmptyFields)->active;
        BROWSE_SUBDIR               = GTK_TOGGLE_BUTTON(BrowseSubdir)->active;
        OPEN_SELECTED_BROWSER_NODE  = GTK_TOGGLE_BUTTON(OpenSelectedBrowserNode)->active;
        KEEP_TREE_BROWSER_IN_MEMORY = GTK_TOGGLE_BUTTON(KeepTreeBrowserInMemory)->active;
        SAVE_CONFIG_BEFORE_EXIT     = GTK_TOGGLE_BUTTON(SaveConfigBeforeExit)->active;

        /* User interface */
        SET_MAIN_WINDOW_SIZE      = GTK_TOGGLE_BUTTON(SetMainWindowSize)->active;
        MAIN_WINDOW_HEIGHT        = atoi(gtk_entry_get_text_1(MainWindowHeight));
        MAIN_WINDOW_WIDTH         = atoi(gtk_entry_get_text_1(MainWindowWidth));
        SET_PANE_HANDLE_POSITION1 = GTK_TOGGLE_BUTTON(SetPaneHandlePosition1)->active;
        PANE_HANDLE_POSITION1     = atoi(gtk_entry_get_text_1(PaneHandlePosition1));
        SET_PANE_HANDLE_POSITION2 = GTK_TOGGLE_BUTTON(SetPaneHandlePosition2)->active;
        PANE_HANDLE_POSITION2     = atoi(gtk_entry_get_text_1(PaneHandlePosition2));
        SHOW_HEADER_INFO          = GTK_TOGGLE_BUTTON(ShowHeaderInfos)->active;
        SHOW_COMMAND_BUTTONS      = GTK_TOGGLE_BUTTON(ShowCommandButtons)->active;

        SHOW_SPIN_BUTTON        = GTK_TOGGLE_BUTTON(ShowSpinButton)->active;
        SHOW_GO_BUTTON          = GTK_TOGGLE_BUTTON(ShowGoButton)->active;
        SHOW_FIRST_BUTTON       = GTK_TOGGLE_BUTTON(ShowFirstButton)->active;
        SHOW_PREV_BUTTON        = GTK_TOGGLE_BUTTON(ShowPrevButton)->active;
        SHOW_NEXT_BUTTON        = GTK_TOGGLE_BUTTON(ShowNextButton)->active;
        SHOW_LAST_BUTTON        = GTK_TOGGLE_BUTTON(ShowLastButton)->active;
        SHOW_SCAN_BUTTON        = GTK_TOGGLE_BUTTON(ShowScanButton)->active;
        SHOW_SCAN_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowScanAllButton)->active;
        SHOW_REMOVE_BUTTON      = GTK_TOGGLE_BUTTON(ShowRemoveButton)->active;
        SHOW_REMOVE_ALL_BUTTON  = GTK_TOGGLE_BUTTON(ShowRemoveAllButton)->active;
        SHOW_UNDO_BUTTON        = GTK_TOGGLE_BUTTON(ShowUndoButton)->active;
        SHOW_UNDO_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowUndoAllButton)->active;
        SHOW_REDO_BUTTON        = GTK_TOGGLE_BUTTON(ShowRedoButton)->active;
        SHOW_REDO_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowRedoAllButton)->active;
        SHOW_SAVE_BUTTON        = GTK_TOGGLE_BUTTON(ShowSaveButton)->active;
        SHOW_SAVE_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowSaveAllButton)->active;
        SHOW_QUIT_BUTTON        = GTK_TOGGLE_BUTTON(ShowQuitButton)->active;
        
        USE_COLORED_TOOLTIPS    = GTK_TOGGLE_BUTTON(UseColoredTooltips)->active;

        /* Misc */
        REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = GTK_TOGGLE_BUTTON(ReplaceIllegalCharactersInFilename)->active;
        DATE_AUTO_COMPLETION          = GTK_TOGGLE_BUTTON(DateAutoCompletion)->active;
        NUMBER_TRACK_FORMATED         = GTK_TOGGLE_BUTTON(NumberTrackFormated)->active;
        USE_CHARACTER_SET_TRANSLATION = GTK_TOGGLE_BUTTON(UseCharacterSetTranslation)->active;
        FILE_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(FileCharacterSetEntry));
        USER_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(UserCharacterSetEntry));

        MESSAGE_BOX_POSITION_NONE   = GTK_TOGGLE_BUTTON(MessageBoxPositionNone)->active;
        MESSAGE_BOX_POSITION_CENTER = GTK_TOGGLE_BUTTON(MessageBoxPositionCenter)->active;
        MESSAGE_BOX_POSITION_MOUSE  = GTK_TOGGLE_BUTTON(MessageBoxPositionMouse)->active;

        /* Scanner */
        CONVERT_UNDERSCORE_AND_P20_INTO_SPACE = GTK_TOGGLE_BUTTON(ConvertUnderscoreAndP20IntoSpace)->active;
        CONVERT_SPACE_INTO_UNDERSCORE         = GTK_TOGGLE_BUTTON(ConvertSpaceIntoUnderscore)->active;

        OVERWRITE_TAG_FIELD = GTK_TOGGLE_BUTTON(OverwriteTagField)->active;
        SET_DEFAULT_COMMENT = GTK_TOGGLE_BUTTON(SetDefaultComment)->active;
        DEFAULT_COMMENT     = g_strdup(gtk_entry_get_text_1(DefaultComment));

        SCAN_WINDOW_ON_TOP          = GTK_TOGGLE_BUTTON(ScanWindowOnTop)->active;
        SET_SCANNER_WINDOW_POSITION = GTK_TOGGLE_BUTTON(SetScannerWindowPosition)->active;
        SCANNER_WINDOW_X            = atoi(gtk_entry_get_text_1(ScannerWindowX));
        SCANNER_WINDOW_Y            = atoi(gtk_entry_get_text_1(ScannerWindowY));

        /* Confirmation */
        CONFIRM_BEFORE_EXIT    = GTK_TOGGLE_BUTTON(ConfirmBeforeExit)->active;
        CONFIRM_WRITE_TAG      = GTK_TOGGLE_BUTTON(ConfirmWriteTag)->active;
        CONFIRM_RENAME_FILE    = GTK_TOGGLE_BUTTON(ConfirmRenameFile)->active;
        CONFIRM_WRITE_PLAYLIST = GTK_TOGGLE_BUTTON(ConfirmWritePlayList)->active;

        /* Get the last visible notebook page */
        OPTIONS_NOTEBOOK_PAGE = gtk_notebook_get_current_page(GTK_NOTEBOOK(OptionsNoteBook));

        /* Parameters and variables of Scanner Window are in "scan.c" file */

        /*
         * Changes in user interface
         */
        if (SHOW_HEADER_INFO) gtk_widget_show_all(HeaderInfosTable);
        else                  gtk_widget_hide_all(HeaderInfosTable);

        if (SHOW_COMMAND_BUTTONS) gtk_widget_show_all(CommandArea);
        else                      gtk_widget_hide_all(CommandArea);
        /* Must be placed after "SHOW_COMMAND_BUTTONS" cause it can show ALL */
        Update_Button_Visibility_Into_Command_Bar();

        /* Update state of check-menu-item into main/popup menu to browse subdirs */
        Check_Menu_Item_Update_Browse_Subdir();

        if (ScannerWindow)
        {
            gtk_widget_hide(ScannerWindow);
            if (SCAN_WINDOW_ON_TOP)
                gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),GTK_WINDOW(MainWindow));
            else
                gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),NULL);
            gtk_widget_show(ScannerWindow);

            Scan_Set_Scanner_Window_Init_Position();
        }
        
        if (WritePlaylistWindow)
        {
            /* Playlist */
            PLAYLIST_NAME          = g_strdup(gtk_entry_get_text_1(PlayListNameEntry));
            PLAYLIST_USE_NAME      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_name));
            PLAYLIST_USE_DIR_NAME  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name));
            PLAYLIST_FULL_PATH     = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_full_path));
            PLAYLIST_RELATIVE_PATH = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_relative_path));
        }

/*        if (SET_WINDOW_SIZE)    gdk_window_resize(MainWindow->window,WINDOW_HEIGHT,WINDOW_WIDTH);
 *        else            gdk_window_resize(MainWindow->window,0,0);
 */

    }
    /* Config part of scanner window */
    Scan_Apply_Changes();
}

void Save_Changes (void)
{
    Apply_Changes();
    Save_Config_To_File();
}


void Save_Config_Now (void)
{
    Save_Changes();
    Statusbar_Message(_("Configuration saved"),TRUE);
}


/*
 * Write the config file
 */
void Save_Config_To_File (void)
{
    gchar *file_path = NULL;
    FILE *file;

    /* The file to write */
    if (!HOME_VARIABLE) return;
    file_path = g_strconcat(HOME_VARIABLE,
                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                            CONFIG_FILE,NULL);
    
    if ( Create_Easytag_Directory()==0 || (file=fopen(file_path,"w+"))==0 )
    {
        g_print(_("ERROR: Can't write config file: %s (%s)\n\a"),file_path,g_strerror(errno));
    }else
    {
        gint ConfigVarListLen = sizeof(Config_Variables)/sizeof(tConfigVariable);
        gint i;
        gchar *data = NULL;
        for (i=0; i<ConfigVarListLen; i++)
        {
            switch(Config_Variables[i].type)
            {
                case CV_TYPE_INT:
                case CV_TYPE_BOOL:
                {
                    data = g_strdup_printf("%s=%i\n",Config_Variables[i].name,
                                                     *(int *)Config_Variables[i].var);
                    fwrite(data,strlen(data),1,file);
                    //g_print("# var:%d\n",*(int *)Config_Variables[i].var);
                    g_free(data);
                    break;
                }
                case CV_TYPE_STRING:
                {
                    /* Doesn't write datum if empty */
                    if ( (*(char **)Config_Variables[i].var)==NULL ) break;
                    
                    data = g_strdup_printf("%s=%s\n",Config_Variables[i].name,
                                                     *(char **)Config_Variables[i].var);
                    fwrite(data,strlen(data),1,file);
                    //g_print("# var:'%s' => %s",*(char **)Config_Variables[i].var,data);
                    g_free(data);
                    break;
                }
                default:
                {
                    g_print("ERROR: Can't save: type of config variable not supported "
                            "for '%s'!\n\a",Config_Variables[i].name);
                    break;
                }
            }
        }
        fclose(file);
    }
    if (file_path) g_free(file_path);

    //Display_Config();
}


/*
 * Parse lines read (line as <var_description>=<value>) and load the values 
 * into the corresponding config variables.
 */
void Set_Config (gchar *line)
{
    gchar *var_descriptor;
    gchar *var_value;
    gint ConfigVarListLen;
    gint i;

    if (*line=='\n' || *line=='#') return;

    /* Cut string */
    var_descriptor = (gchar*)strtok(line,"=");
    var_value      = (gchar*)strtok(NULL,"=");
    //g_print("\nstr1:'%s',\t str2:'%s'",var_descriptor,var_value);

    ConfigVarListLen = sizeof(Config_Variables)/sizeof(tConfigVariable);
    for (i=0; i<ConfigVarListLen; i++)
    {
        if (!strcmp(Config_Variables[i].name,var_descriptor))
        {
            switch(Config_Variables[i].type)
            {
                case CV_TYPE_INT:
                case CV_TYPE_BOOL:
                {
                          *(int *)Config_Variables[i].var = strtol(var_value, NULL, 10);
                    break;
                }
                case CV_TYPE_STRING:
                {
                    if (!var_value)
                    {
                        *(char **)Config_Variables[i].var = NULL;
                        //g_print("\nConfig File Warning: Field of '%s' has no value!\n",var_descriptor);
                    } else
                    {
                        if ( *(char **)Config_Variables[i].var != NULL )
                            g_free(*(char **)Config_Variables[i].var);
                        *(char **)Config_Variables[i].var = g_malloc(strlen(var_value)+1);
                            strcpy( *(char **)Config_Variables[i].var,var_value );
                    }
                    break;
                }
                default:
                {
                    g_print("ERROR: Can't read: type of config variable not supported "
                            "for '%s'!\n\a",Config_Variables[i].name);
                    break;
                }
            }
        }
    }
}


/*
 * Read config from config file
 */
void Read_Config (void)
{
    gchar *file_path = NULL;
    FILE *file;
    gchar buffer[MAX_STRING_LEN];

    /* The file to write */
    if (!HOME_VARIABLE) return;
    file_path = g_strconcat(HOME_VARIABLE,
                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                            CONFIG_FILE,NULL);
    
    if ( (file=fopen(file_path,"r"))==0 )
    {
        g_print(_("\nCan't open configuration file '%s' (%s)\n"),file_path,g_strerror(errno));
        g_print(_("Loading default configuration...\n"));
    }else
    {
        while(fgets(buffer,sizeof(buffer),file))
        {
            if (buffer[strlen(buffer)-1]=='\n')
                buffer[strlen(buffer)-1]='\0';
            Set_Config(buffer);
        }
        fclose(file);
    }
    if (file_path) g_free(file_path);
}


/*
 * Display values in config variables
 * For debuging only!
 */
void Display_Config (void)
{
    gchar *file_path = NULL;
    FILE *file;

    /* The file to write */
    if (!HOME_VARIABLE) return;
    file_path = g_strconcat(HOME_VARIABLE,
                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                            CONFIG_FILE,NULL);
    
    if ( (file=fopen(file_path,"r"))==0 )
    {
        g_print("Can't open configuration file '%s' (%s)\n",file_path,g_strerror(errno));
    }else
    {
        gint ConfigVarListLen = sizeof(Config_Variables)/sizeof(tConfigVariable);
        gint i;
        
        g_print("\n## Current Config ##");
        for (i=0; i<ConfigVarListLen; i++)
        {
            switch(Config_Variables[i].type)
            {
                case CV_TYPE_INT:
                case CV_TYPE_BOOL:
                {
                    g_print("\n%d: %s=%d",i,Config_Variables[i].name,
                                     *(int*)Config_Variables[i].var);
                    break;
                }
                case CV_TYPE_STRING:
                {
                    g_print("\n%d: %s=%s",i,Config_Variables[i].name,
                                     *(char**)Config_Variables[i].var);
                    break;
                }
                default:
                {
                    g_print("NOT IMPLEMENTED (Save_Config)!! \n\a");
                    break;
                }
            }
        }
        g_print("\n## End Current Config ##\n");
        fclose(file);
    }
    if (file_path) g_free(file_path);
}





/*
 * Save a glist list into file
 * Note: if list==NULL, then the file is empty (so the default list will be load at the next reading)
 */
void Save_Glist_To_File (GList *list, gchar *filename)
{
    gchar *file_path = NULL;
    FILE *file;
    gchar *data = NULL;
    GList *tmplist = NULL;

    /* The file to write */
    if (!HOME_VARIABLE) return;
    file_path = g_strconcat(HOME_VARIABLE,
                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                            filename,NULL);
    
    if ( Create_Easytag_Directory()==0 || (file=fopen(file_path,"w+"))==0 )
    {
        g_print(_("ERROR: Can't write list to file: %s (%s)\n\a"),file_path,g_strerror(errno));
    }else
    {
        tmplist = g_list_first (list);
        while (tmplist)
        {
            /*g_print("Writing data:'%s'\n",(gchar *)tmplist->data);*/
            data = g_strdup_printf("%s\n",(gchar *)tmplist->data);
            if (data)
            {
                fwrite(data,strlen(data),1,file);
                g_free(data);
            }
            tmplist = g_list_next (tmplist);
        }
        fclose(file);
    }
    if (file_path) g_free(file_path);
}


/*
 * Load a glist from file passed in parameter
 */
GList *Create_Glist_From_File (gchar *filename)
{
    GList *list = NULL;
    gchar *file_path = NULL;
    FILE *file;
    gchar buffer[MAX_STRING_LEN];

    /* The file to write */
    if (!filename || !HOME_VARIABLE) return NULL;
    file_path = g_strconcat(HOME_VARIABLE,
                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                            filename,NULL);
    
    if ( (file=fopen(file_path,"r"))==0 )
    {
        g_print(_("Can't open file '%s' (%s)\n"),file_path,g_strerror(errno));
    }else
    {
        gchar *data = NULL;

        while(fgets(buffer,sizeof(buffer),file))
        {
            if (buffer[strlen(buffer)-1]=='\n')
                buffer[strlen(buffer)-1]='\0';
            data = g_strdup(buffer);
            if (data && strlen(data)>0)
                list = g_list_append(list,data);
        }
        fclose(file);
    }
    if (file_path) g_free(file_path);
    return list;
}


/*
 * Functions for writing and reading list of 'Fill Tag' masks
 */
GList *Load_Scan_Tag_Masks_List (void)
{
    return Create_Glist_From_File(SCAN_TAG_MASKS_FILE);
}
void Save_Scan_Tag_Masks_List (GList *list)
{
    Save_Glist_To_File(list,SCAN_TAG_MASKS_FILE);
}


/*
 * Functions for writing and reading list of 'Rename File' masks
 */
GList *Load_Rename_File_Masks_List (void)
{
    return Create_Glist_From_File(RENAME_FILE_MASKS_FILE);
}
void Save_Rename_File_Masks_List (GList *list)
{
    Save_Glist_To_File(list,RENAME_FILE_MASKS_FILE);
}



/*
 * Functions for writing and reading list of 'DefaultPathToMp3' combobox
 */
GList *Load_Default_Path_To_MP3_List (void)
{
    return Create_Glist_From_File(DEFAULT_PATH_TO_MP3_HISTORY_FILE);
}
void Save_Default_Path_To_MP3_List (GList *list)
{
    Save_Glist_To_File(list,DEFAULT_PATH_TO_MP3_HISTORY_FILE);
}

/*
 * Functions for writing and reading list of 'DefaultComment' combobox
 */
GList *Load_Default_Tag_Comment_Text_List (void)
{
    return Create_Glist_From_File(DEFAULT_TAG_COMMENT_HISTORY_FILE);
}
void Save_Default_Tag_Comment_Text_List (GList *list)
{
    Save_Glist_To_File(list,DEFAULT_TAG_COMMENT_HISTORY_FILE);
}

/*
 * Functions for writing and reading list of 'TranslationTablePath' combobox
 */
GList *Load_Translation_Table_Path_List (void)
{
    return Create_Glist_From_File(TRANSLATION_TABLE_PATH_HISTORY_FILE);
}
void Save_Translation_Table_Path_List (GList *list)
{
    Save_Glist_To_File(list,TRANSLATION_TABLE_PATH_HISTORY_FILE);
}

/*
 * Functions for writing and reading list of 'BrowserEntry' combobox
 */
GList *Load_Path_Entry_List (void)
{
    return Create_Glist_From_File(PATH_ENTRY_HISTORY_FILE);
}
void Save_Path_Entry_List (GList *list)
{
    Save_Glist_To_File(list,PATH_ENTRY_HISTORY_FILE);
}

/*
 * Functions for writing and reading list of 'PlayListNameEntry' combobox
 */
GList *Load_Play_List_Name_List (void)
{
    return Create_Glist_From_File(PLAY_LIST_NAME_HISTORY_FILE);
}
void Save_Play_List_Name_List (GList *list)
{
    Save_Glist_To_File(list,PLAY_LIST_NAME_HISTORY_FILE);
}

/*
 * Functions for writing and reading list of combobox to run program (browser)
 */
GList *Load_Run_Program_List (void)
{
    return Create_Glist_From_File(RUN_PROGRAM_HISTORY_FILE);
}
void Save_Run_Program_List (GList *list)
{
    Save_Glist_To_File(list,RUN_PROGRAM_HISTORY_FILE);
}



/*
 * Create the directory used by EasyTAG to store files for each user.
 * If the directory already exists, does nothing and returns 1.
 * If unable to create the directory, returns 0.
 */
gboolean Create_Easytag_Directory (void)
{
    gchar *easytag_path = NULL;
    DIR *dir;

    if (!HOME_VARIABLE)
    {
        g_print(_("ERROR: The environment variable HOME is not defined!\n\n"));
        return 0;
    }

    /* Directory to create (if doesn't exists) with absolute path */
    easytag_path = g_strconcat(HOME_VARIABLE,
                               HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                               EASYTAG_DIR,
                               EASYTAG_DIR[strlen(EASYTAG_DIR)-1]!='/'?"/":"",
                               NULL);

    if ( (dir=opendir(easytag_path)) == NULL )
    {
        if ( (mkdir(easytag_path,S_IRWXU|S_IXGRP|S_IRGRP)) == -1)
        {
            g_print(_("ERROR: Can't create directory '%s' (%s)!\n"),easytag_path,g_strerror(errno));
            return 0;          
        }
    }else
    {
        closedir(dir);
    }
    
    if (easytag_path)
        g_free(easytag_path);

    return 1;
}
